// (c) by Sitsanlis Ilias All rights reserved

var	vectorConeLength=3.5;
var	lMin=10;

THREE.myVector = function ( dir, origin, length, width, hex ) {
	THREE.Object3D.call( this );
	if ( length === undefined ) length = 20;
	if ( hex === undefined ) hex = 0xffffff;
	if ( width === undefined) width=0.6

	//vectorConeLength=width*5

	var cylinderGeometry = new THREE.CylinderGeometry( width, width, 1, 20, 1 );
	this.cylinder = new THREE.Mesh( cylinderGeometry, new THREE.MeshBasicMaterial( { color: hex, transparent:false } ) );
	this.cylinder.position.set( 0, 1/2, 0 );
	this.add( this.cylinder );

	var coneGeometry = new THREE.CylinderGeometry( 0, width*2.5, vectorConeLength, 20, 1 );
	this.cone = new THREE.Mesh( coneGeometry, new THREE.MeshBasicMaterial( { color: hex, transparent:false } ) );
	this.cone.position.set( 0, 1, 0 );
	this.add( this.cone );

	if ( origin instanceof THREE.Vector3 ) this.position = origin;
	this.setDirection( dir );
	this.setLength( length );
};

THREE.myVector.prototype = Object.create( THREE.Object3D.prototype );

THREE.myVector.prototype.setDirection = function ( dir ) {
    var d = THREE.myVector.__v1.copy( dir ).normalize();
    if ( d.y > 0.999 ) {
        this.rotation.set( 0, 0, 0 );
    } else if ( d.y < - 0.999 ) {
        this.rotation.set( Math.PI, 0, 0 );
    } else {
		var axis = THREE.myVector.__v2.set( d.z, 0, - d.x ).normalize();
		var radians = Math.acos( d.y );
		var quaternion = THREE.myVector.__q1.setFromAxisAngle( axis, radians );
		this.rotation.setEulerFromQuaternion( quaternion, this.eulerOrder );
	}
};

THREE.myVector.prototype.setLength = function ( length ) {
	if (Math.abs(length)<=2) {
		this.cone.visible = false;
		this.cylinder.visible=false;
	} else {
		this.cone.visible = true;
		this.cylinder.visible=true;
	}

	if (length>=lMin) {
		this.cone.scale.set(1,1,1)
		this.cylinder.scale.set(1,length-vectorConeLength/2,1);
		this.cone.position.set(0,(length-vectorConeLength/2),0);
		this.cylinder.position.set(0,(length-vectorConeLength/2)/2,0);
	} else {
		this.cone.scale.set(1,length/lMin,1);
		this.cylinder.scale.set(1,length-(length*vectorConeLength/lMin)/2,1);
		this.cone.position.set(0,length-(length*vectorConeLength/lMin)/2,0);
		this.cylinder.position.set(0,(length-(length*vectorConeLength/lMin)/2)/2,0);
	}
	//this.cylinder.scale.set(1,length,1);
	
};

THREE.myVector.prototype.setColor = function ( hex, opacity ) {
	if ( opacity === undefined ) opacity = 1;

	this.cylinder.material.color.setHex( hex );
	this.cylinder.material.opacity=opacity

	this.cone.material.color.setHex( hex );
	this.cone.material.opacity=opacity
};



THREE.myVector.__v1 = new THREE.Vector3();
THREE.myVector.__v2 = new THREE.Vector3();
THREE.myVector.__q1 = new THREE.Quaternion();

function alignCube(target, dir, rot) {
    var up = new THREE.Vector3(1, 0, 0);
    var angle = Math.acos(up.dot(dir));
    var axis = new THREE.Vector3();
    axis.cross(up, dir);
    axis.normalize();
    var position = THREE.Matrix4.translationMatrix( target.position.x, target.position.y, target.position.z );
    var rotate = THREE.Matrix4.rotationAxisAngleMatrix(axis, angle);
    var revolve = THREE.Matrix4.rotationAxisAngleMatrix(dir, rot);
    var scale = THREE.Matrix4.scaleMatrix( 1, 1, 1 );
    revolve.multiplySelf(rotate);
    position.multiplySelf(revolve);
    target.matrix = position;
}